@@ -234,6 +234,17 @@ class WorkflowEnabled(base.BaseWorkflowEnabled):
 class BaseTransitionLog(models.Model):
     """Abstract model for a minimal database logging setup.
 
+    Class attributes:
+        MODIFIED_OBJECT_FIELD (str): name of the field storing the modified
+            object.
+        EXTRA_LOG_ATTRIBUTES ((db_field, kwarg, default) list): Describes extra
+            transition kwargs to store:
+            - db_field is the name of the attribute where data should be stored
+            - kwarg is the name of the keyword argument of the transition to
+              record
+            - default is the default value to store if no value was provided for
+              kwarg in the transition's arguments
+
     Attributes:
         modified_object (django.db.model.Model): the object affected by this
             transition.
@@ -242,16 +253,8 @@ class BaseTransitionLog(models.Model):
         transition (str): The name of the transition being performed.
         timestamp (datetime): The time at which the Transition was performed.
     """
-
-    content_type = models.ForeignKey(ct_models.ContentType,
-                                     verbose_name=_(u"Content type"),
-                                     related_name="workflow_object",
-                                     blank=True, null=True)
-    content_id = models.PositiveIntegerField(_(u"Content id"),
-        blank=True, null=True, db_index=True)
-    modified_object = generic.GenericForeignKey(
-            ct_field="content_type",
-            fk_field="content_id")
+    MODIFIED_OBJECT_FIELD = ''
+    EXTRA_LOG_ATTRIBUTES = ()
 
     transition = models.CharField(_(u"transition"), max_length=255,
         db_index=True)
@@ -268,9 +271,46 @@ class BaseTransitionLog(models.Model):
         verbose_name_plural = _(u'XWorkflow transition logs')
         abstract = True
 
+    def get_modified_object(self):
+        if self.MODIFIED_OBJECT_FIELD:
+            return getattr(self, self.MODIFIED_OBJECT_FIELD, None)
+        return None
+
     def __unicode__(self):
-        return u'%r: %s -> %s at %s' % (self.modified_object, self.from_state,
-            self.to_state, self.timestamp.isoformat())
+        return u'%r: %s -> %s at %s' % (self.get_modified_object(),
+            self.from_state, self.to_state, self.timestamp.isoformat())
+
+
+class GenericTransitionLog(BaseTransitionLog):
+    """Abstract model for a minimal database logging setup.
+
+    Specializes BaseTransitionLog to use a GenericForeignKey.
+
+    Attributes:
+        modified_object (django.db.model.Model): the object affected by this
+            transition.
+        from_state (str): the name of the origin state
+        to_state (str): the name of the destination state
+        transition (str): The name of the transition being performed.
+        timestamp (datetime): The time at which the Transition was performed.
+    """
+    MODIFIED_OBJECT_FIELD = 'modified_object'
+
+    content_type = models.ForeignKey(ct_models.ContentType,
+                                     verbose_name=_(u"Content type"),
+                                     related_name="workflow_object",
+                                     blank=True, null=True)
+    content_id = models.PositiveIntegerField(_(u"Content id"),
+        blank=True, null=True, db_index=True)
+    modified_object = generic.GenericForeignKey(
+            ct_field="content_type",
+            fk_field="content_id")
+
+    class Meta:
+        ordering = ('-timestamp', 'transition')
+        verbose_name = _(u'XWorkflow transition log')
+        verbose_name_plural = _(u'XWorkflow transition logs')
+        abstract = True
 
 
 def get_default_log_model():
@@ -302,19 +342,25 @@ class Workflow(base.Workflow):
     #: Save log to this django model (name of the model)
     log_model = get_default_log_model()
 
+    #: Save log to this django model (actual class)
+    log_model_class = None
+
     def __init__(self, *args, **kwargs):
         # Fetch 'log_model' if overridden.
         log_model = kwargs.pop('log_model', self.log_model)
 
+        # Fetch 'log_model_class' if overridden.
+        log_model_class = kwargs.pop('log_model_class', self.log_model_class)
+
         super(Workflow, self).__init__(*args, **kwargs)
 
         self.log_model = log_model
-        self.log_model_class = None
+        self.log_model_class = log_model_class
 
     def _get_log_model_class(self):
         """Cache for fetching the actual log model object once django is loaded.
 
-        Otherwise, import conflict occur: WorkflowEnabled import <log_model>
+        Otherwise, import conflict occur: WorkflowEnabled imports <log_model>
         which tries to import all models to retrieve the proper model class.
         """
         if self.log_model_class is not None:
@@ -326,14 +372,18 @@ class Workflow(base.Workflow):
 
     def db_log(self, transition, from_state, instance, *args, **kwargs):
         """Logs the transition into the database."""
-        user = kwargs.pop('user', None)
         if self.log_model:
             model_class = self._get_log_model_class()
+
+            extras = {}
+            for db_field, transition_arg, default in model_class.EXTRA_LOG_ATTRIBUTES:
+                extras[db_field] = kwargs.get(transition_arg, default)
+
             model_class.objects.create(modified_object=instance,
                                        transition=transition.name,
                                        from_state=from_state.name,
                                        to_state=transition.target.name,
-                                       user=user)
+                                       **extras)
 
     def log_transition(self, transition, from_state, instance, *args, **kwargs):
         """Generic transition logging."""
